// Copyright (c) 2015, AppFoundry. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

// TODO: Put public facing types in this file.

library d_calendar.base;

import "dart:html";
import "dart:core";
import 'd_day.dart';

class Calendar
{ 
  Calendar(DivElement container)
  {  
    _generateHtml(container);  
    _days = new Map();
    DateTime now = new DateTime.now(); 
    int weekday = now.weekday;
    
    for (int i = 1; i < 8; i++)
    {
      _currentWeek.add(now.add(new Duration(days:i-weekday)));     
    }

    for (int i = 0; i < 7; i++)
    {
      _divWeekdays[i].querySelector(".dayofweek").innerHtml = _weekdays[i];
    }    

    refresh();
    _next.onClick.listen((_) => refresh(1));
    _previous.onClick.listen((_) => refresh(-1));
  }
  
  void clear()
  {
    _days.clear();
    refresh();
  }
     
  void refresh([int week_offset = 0]) 
  {   
    _cWeekOffset += week_offset;
    
    /// Update current week
    for (int i = 0; i <_currentWeek.length; i++)
    {
      _currentWeek[i] = _currentWeek[i].add(new Duration(days:7*week_offset));
    }

    for (int i = 0; i < 7; i++)
    {
      DateTime date = _currentWeek[i];
      _divWeekdays[i].querySelector(".date").innerHtml = date.day.toString();
      _divWeekdays[i].querySelector(".month").innerHtml = _months[date.month - 1];
      DivElement highlight = _divWeekdays[i].querySelector(".content").querySelector(".highlight");
      highlight.children.clear();
      _divWeekdays[i].dataset["date"] = date.toIso8601String();
                        
      if (_days.containsKey(date.year))
      {        
        if (_days[date.year].containsKey(date.month))
        {
          Map<int, Day> week = _days[date.year][date.month];
          if (week.containsKey(date.day)) highlight.children.add(week[_currentWeek[i].day].content);            
        }
      }
    }
    _week.innerHtml = calcWeekOfYear(_cWeekOffset).toString();
  }
  
  int calcWeekOfYear(int week_offset)
  {
    DateTime current = new DateTime.now().add(new Duration(days:7*week_offset));
    _year.innerHtml = current.year.toString();
    /// First day of year is not always monday! But we assume every week is 7 days. Offset the first week by number of days since monday.
    DateTime firstDayOfYear = new DateTime(current.year);     
    _daysPerMonth[1] = 31 + firstDayOfYear.weekday;    
    _daysPerMonth[2] = (current.year % 4 == 0 && current.year % 100 == 0 && current.year % 400 == 0) ? 29 : 28; /// Leap year
    _daysPerMonth[3] = 31;
    _daysPerMonth[4] = 30;
    _daysPerMonth[5] = 31;
    _daysPerMonth[6] = 30;
    _daysPerMonth[7] = 31;
    _daysPerMonth[8] = 31;
    _daysPerMonth[9] = 30;
    _daysPerMonth[10] = 31;
    _daysPerMonth[11] = 30;
    _daysPerMonth[12] = 31;
    
    int currentDayOfYear = 0;
    
    for (int i = 1; i < current.month; i++)
    {
      currentDayOfYear += _daysPerMonth[i];
    }
    currentDayOfYear += current.day;
    return (currentDayOfYear.toDouble() / 7).ceil();
  }
  
  int get currentWeek => calcWeekOfYear(_cWeekOffset);
  
  Day getDay(int year, int month, int day)
  {  
    DateTime date = new DateTime(year, month, day);
    if (date.day != day || date.year != year || date.month != month) throw(new StateError("Date is not valid"));      
    if (!_days.containsKey(year)) _days[year] = new Map();     
    if (!_days[year].containsKey(month)) _days[year][month] = new Map();
    if (!_days[year][month].containsKey(day)) _days[year][month][day] = new Day(date);     
    return _days[year][month][day];
  }
  
  Day getDayParse(String date)
  {
    DateTime d = DateTime.parse(date);
    return getDay(d.year, d.month, d.day);
  }
  
  void _generateHtml(DivElement container)
  {
    if (container == null) throw new ArgumentError.notNull("container");
            
    DivElement buttonRow = new DivElement();    
    DivElement calendarRow = new DivElement();
    buttonRow.className = calendarRow.className = "row";
    
    _next = new ButtonElement();
    _previous = new ButtonElement();
    _previous.className = "small-1 small-offset-5 columns";
    _next.className = "small-1 columns end";
    _next.innerHtml = "&raquo;";
    _previous.innerHtml = "&laquo;";
    buttonRow.children.add(_previous);
    buttonRow.children.add(_next);
        
    DivElement monthInfo = new DivElement();
    monthInfo.className = "small-6 medium-3 large-1 large-offset-2 columns weekday";
    DivElement label = new DivElement();
    label.innerHtml = "year";
    _year = new DivElement();   
    _year.style.fontSize = "2em";
    _year.style.lineHeight = "0.75em";
    _week = new DivElement();
    monthInfo.children.add(label);
    monthInfo.children.add(_year);
    monthInfo.children.add(_week);    
    monthInfo.children.add(new HRElement());
    monthInfo.children.last.style.margin = "0.5em 0 2px 0";
    calendarRow.children.add(monthInfo);
    monthInfo.style.paddingLeft = monthInfo.style.paddingRight = "0";
    monthInfo.style.textAlign = "center";
    monthInfo.style.border = "1px solid #ddd";
    
    /// add hours to the first column ("monthinfo"), excluding hours printed
    DivElement hours = new DivElement();
    for (int i = 6; i < 22; i++)
    {
      DivElement hour = new DivElement();
      hour.style.height = "30px";
      hour.style.position = "relative";
      //hour.style.backgroundColor = (i % 2 == 0) ? "#fff" : "#f3f3f3";
      hours.children.add(hour);      
    }
    monthInfo.children.add(hours);
    
    for (int i = 0; i < 7; i++)
    {
      DivElement weekday = new DivElement();
      weekday.className = "small-6 medium-3 large-1 columns weekday";
      weekday.id = _weekdays[i];
      weekday.style.paddingLeft = weekday.style.paddingRight = "0";
      weekday.style.textAlign = "center";
      weekday.style.border = "1px solid #ddd";            
      
      List<Element> children = new List();
      children.add(new DivElement());
      children.last.className = "dateinfo";
      
      DivElement day = new DivElement();
      day.className = "dayofweek";
      children.add(day);
      
      DivElement date = new DivElement();
      date.className = "date";
      date.style.fontSize = "2em";
      date.style.lineHeight = "0.75em";
      children.add(date);
      
      DivElement month = new DivElement();
      month.className = "month";
      children.add(month);

      children.add(new HRElement());
      children.last.style.margin = "0.5em 0 2px 0";
      
      // Add hours to the first column ("monthinfo"), including hours printed
      DivElement hours = new DivElement();
      for (int i = 6; i < 22; i++)
      {
        DivElement hour = new DivElement();
        hour.style.height = "30px";
        hour.style.position = "relative";
        hour.style.lineHeight = hour.style.fontSize = "14px";
        hour.style.color = "#aaa";
        hour.innerHtml = (i < 10) ? "0" + i.toString() + ".00" : i.toString() + ":00";
        hour.style.backgroundColor = (i % 2 == 0) ? "#fff" : "#f3f3f3";
        hours.children.add(hour);      
      }
      
      DivElement highlights = new DivElement();
      highlights.className = "highlight";
      DivElement dayContent = new DivElement();
      dayContent.className = "content";
      dayContent.style.position = "relative";
      dayContent.children.add(hours);
      dayContent.children.add(highlights);      
      children.add(dayContent);
            
      children.forEach((Element e) => weekday.children.add(e));     
      _divWeekdays.add(weekday);
      calendarRow.children.add(weekday);
    }   
    calendarRow.children.last.classes.add("end");
        
    container.children.add(buttonRow);
    container.children.add(calendarRow);
  }
  
  List<DateTime> _currentWeek = new List();
  List<DivElement> _divWeekdays = new List();
  DivElement _year;
  DivElement _week;
  List<String> _weekdays = ["mon","tue","wed","thu","fri","sat","sun"];
  List<String> _months = ["januari","february","march","april","may","june","july","august","september","october","november","december"];
  Map<int, int> _daysPerMonth = new Map(); 
  ButtonElement _next;
  ButtonElement _previous;
  int _cWeekOffset = 0;
  Map<int, Map<int, Map<int, Day>>> _days; 
}