import "dart:html";

class DaySection
{  
  final String color;
  final String timeStart;
  final String timeEnd;
  const DaySection(this.color, this.timeStart, this.timeEnd);
}


class Day
{
  Day(this._date)
  {
    _content = new DivElement();
  }
  
  void setContent(String title, List<DaySection> sections)
  {    
    _content.children.clear();   
    if (title == null) throw new ArgumentError.notNull("title");
    if (sections == null) throw new ArgumentError.notNull("sections");    
    
    sections.forEach((DaySection section)
    {      
      String day = _date.toIso8601String();
      day = day.substring(0, day.lastIndexOf("-") + 3);
      
      DateTime timeStart = _date.add(DateTime.parse(day + " " + section.timeStart).difference(_date));            
      DateTime timeEnd = _date.add(DateTime.parse(day + " " + section.timeEnd).difference(_date));
      
      int minutes = timeEnd.difference(timeStart).inMinutes;
      int sectionTop = (0.5 * (timeStart.difference(DateTime.parse(day + " 00:00")).inMinutes - 360)).round();
      int sectionHeight = (0.5 * minutes).round();
      
      DivElement sectionElement = new DivElement();
      sectionElement.style.position = "absolute";
      sectionElement.style.top = sectionTop.toString() + "px";
      sectionElement.style.width = "100%";
      sectionElement.style.display = "block";
      sectionElement.style.backgroundColor = section.color;
      sectionElement.style.opacity = "0.3";
      sectionElement.style.height = sectionHeight.toString() + "px";            
      
      String timeStartString = timeStart.toIso8601String();
      String timeEndString = timeEnd.toIso8601String();
      int startIndex = timeStartString.indexOf("T") + 1;            
      sectionElement.dataset["time_start"] = timeStartString.substring(startIndex, startIndex + 5);
      sectionElement.dataset["time_end"] = timeEndString.substring(startIndex, startIndex + 5);                                     
      _content.children.add(sectionElement);      
    });    
  }  
  
  final DateTime _date;
  DivElement _content;
  DivElement get content => _content;
}
